#include "libwidget.h"
#include "ui_libwidget.h"

#include <QDir>
#include <QLibrary>
#include <QList>

#ifdef Q_OS_WIN
#define LIB_SUFFIX "dll"
#else
#define LIB_SUFFIX "so"
#endif

LibWidget::LibWidget(QWidget *parent) :
    QWidget(parent),
    ui(new Ui::LibWidget)
{
    ui->setupUi(this);
    refreshBehaviors();
    refreshGenerators();
    connect(ui->behaviorsRefreshButton, SIGNAL(pressed()), this, SLOT(refreshBehaviors()));
    connect(ui->generatorsRefreshButton, SIGNAL(pressed()), this, SLOT(refreshGenerators()));
    connect(ui->launchButton, SIGNAL(pressed()), this, SLOT(launchSimulation()));
}

LibWidget::~LibWidget()
{
    delete ui;
}

void LibWidget::refreshBehaviors()
{
    while(ui->behaviorsList->count() > 0)
        ui->behaviorsList->takeItem(0);
    m_behaviorList.clear();
    QDir teamDir(QCoreApplication::applicationDirPath());
    if(teamDir.cd("../teams"))
    {
        for(const QFileInfo &info : teamDir.entryInfoList())
        {
            if(info.suffix().compare(LIB_SUFFIX) == 0)
            {
                QLibrary lib(info.absoluteFilePath());
                if(lib.load())
                {
                    BehaviorFunction func = (BehaviorFunction)lib.resolve("think");
                    if(func)
                    {
                        ui->behaviorsList->addItem(info.baseName());
                        m_behaviorList.push_back(func);
                    }
                }
            }
        }
    }
    else
        emit sendError(QString("ERROR : can't open the teams folder.\n"), 5000);
}

void LibWidget::refreshGenerators()
{
    while(ui->generatorsList->count() > 0)
        ui->generatorsList->takeItem(0);
    m_genList.clear();
    QDir genDir(QCoreApplication::applicationDirPath());
    if(genDir.cd("../generators"))
    {
        for(const QFileInfo &info : genDir.entryInfoList())
        {
            if(info.suffix().compare(LIB_SUFFIX) == 0)
            {
                QLibrary lib(info.absoluteFilePath());
                if(lib.load())
                {
                    GenerateFunction func = (GenerateFunction)lib.resolve("generate");
                    if(func)
                    {
                        ui->generatorsList->addItem(info.baseName());
                        m_genList.push_back(func);
                    }
                }
            }
        }
    }
    else
        emit sendError(QString("ERROR : can't open the generators folder.\n"), 5000);
    if(!m_genList.empty())
        ui->generatorsList->item(0)->setSelected(true);
}

void LibWidget::launchSimulation()
{
    GenerateFunction genFunc = m_genList[ui->generatorsList->currentRow()];
    std::vector<BehaviorFunction> behaveFuncs;
    for(QListWidgetItem *item : ui->behaviorsList->selectedItems())
        behaveFuncs.push_back(m_behaviorList[ui->behaviorsList->row(item)]);
    emit createSimulation(genFunc, behaveFuncs);
}
